#include "Adafruit_VL53L0X.h"

// address we will assign if dual sensor is present
#define LOX1_ADDRESS 0x30
#define LOX2_ADDRESS 0x31

// set the pins to shutdown
#define dirPin 13
#define stepPin 3
#define SHT_LOX1 11
#define SHT_LOX2 9
#define Left_switch 7
#define Right_switch 5
bool switch_1_state;
bool switch_2_state;


int interval = 1250;
int intervalValue = 1250;

//check indicates the sense of rotation
int lastDir = 2;
int check = 2;
int goRight = 0;
int goLeft = 0;
unsigned long previousMillis = 0;
// objects for the vl53l0x
Adafruit_VL53L0X lox1 = Adafruit_VL53L0X(); 
Adafruit_VL53L0X lox2 = Adafruit_VL53L0X();

// this holds the measurement
VL53L0X_RangingMeasurementData_t measure1;
VL53L0X_RangingMeasurementData_t measure2;

/*
    Reset all sensors by setting all of their XSHUT pins low for delay(10), then set all XSHUT high to bring out of reset
    Keep sensor #1 awake by keeping XSHUT pin high
    Put all other sensors into shutdown by pulling XSHUT pins low
    Initialize sensor #1 with lox.begin(new_i2c_address) Pick any number but 0x29 and it must be under 0x7F. Going with 0x30 to 0x3F is probably OK.
    Keep sensor #1 awake, and now bring sensor #2 out of reset by setting its XSHUT pin high.
    Initialize sensor #2 with lox.begin(new_i2c_address) Pick any number but 0x29 and whatever you set the first sensor to
 */
void setID() {
  // all reset
  digitalWrite(SHT_LOX1, LOW);    
  digitalWrite(SHT_LOX2, LOW);
  delay(10);
  // all unreset
  digitalWrite(SHT_LOX1, HIGH);
  digitalWrite(SHT_LOX2, HIGH);
  delay(10);

  // activating LOX1 and resetting LOX2
  digitalWrite(SHT_LOX1, HIGH);
  digitalWrite(SHT_LOX2, LOW);

  // initing LOX1
  if(!lox1.begin(LOX1_ADDRESS)) {
    Serial.println(F("Failed to boot first VL53L0X"));
    while(1);
  }
  delay(10);

  // activating LOX2
  digitalWrite(SHT_LOX2, HIGH);
  delay(10);

  //initing LOX2
  if(!lox2.begin(LOX2_ADDRESS)) {
    Serial.println(F("Failed to boot second VL53L0X"));
    while(1);
  }
}

void read_dual_sensors() {

  goLeft = 0;
  goRight = 0;
  
  lox1.rangingTest(&measure1, false); // pass in 'true' to get debug data printout!
  lox2.rangingTest(&measure2, false); // pass in 'true' to get debug data printout!

  // print sensor one reading
  Serial.print(F("1: "));
  if((measure1.RangeStatus != 4) && (measure1.RangeMilliMeter < 3000)){     // if not out of range
    Serial.print(measure1.RangeMilliMeter);
  } 
  else {
    Serial.print(F("Left doesnt see"));
    goRight = 1;
    if(lastDir == 0){
      interval = intervalValue/2;
    }
    lastDir = 1;
  }
  Serial.print(F(" "));

  // print sensor two reading
  Serial.print(F("2: "));
  if((measure2.RangeStatus != 4) && (measure2.RangeMilliMeter < 3000)){     // if not out of range
    Serial.print(measure2.RangeMilliMeter);
  } 
  else {
    Serial.print(F("Right doesnt see"));
    goLeft = 1;
    if(lastDir == 1){
      interval = intervalValue/2;
    }
    lastDir = 0;
  }
  Serial.println();
  
  if(goLeft != goRight){
    check = 1;
  }
  else{check = 0;
       lastDir = 2;
       }
}

void readInputs(){
  if(digitalRead(Left_switch) == HIGH && switch_1_state == 0){
    switch_1_state = 1;
    switch_2_state = 0;
    }
  else if(digitalRead(Right_switch) == HIGH && switch_2_state == 0){
    switch_1_state = 0;
    switch_2_state = 1;
    }
  else if(digitalRead(Left_switch) == LOW && switch_1_state == 1){
    switch_2_state = 0;
    switch_1_state = 0;
  }
  else if(digitalRead(Right_switch) == LOW && switch_2_state == 1){
    switch_2_state = 0;
    switch_1_state = 0;
  }
}

void setup() {
  Serial.begin(115200);

  // wait until serial port opens for native USB devices
  while (! Serial) { delay(1); }

  pinMode(SHT_LOX1, OUTPUT);
  pinMode(SHT_LOX2, OUTPUT);

  Serial.println(F("Shutdown pins inited..."));

  digitalWrite(SHT_LOX1, LOW);
  digitalWrite(SHT_LOX2, LOW);

  Serial.println(F("Both in reset mode...(pins are low)"));
  
  
  Serial.println(F("Starting..."));
  setID();


  // Declare pins as output:
  pinMode(stepPin, OUTPUT);
  pinMode(dirPin, OUTPUT);

  
  // Set the spinning direction CW/CCW:
  digitalWrite(dirPin, LOW);

  pinMode(Left_switch, INPUT_PULLUP);
  pinMode(Right_switch, INPUT_PULLUP);
}


void loop() {

  unsigned long currentMillis = millis();

  if (currentMillis - previousMillis >= interval) {
    previousMillis = currentMillis;
    interval = intervalValue;
    
    read_dual_sensors();
    Serial.println(check);
    Serial.println(goLeft);
    Serial.println(goRight);

    if(goLeft==1 && check == 1){
      Serial.println("go left");
      digitalWrite(dirPin, HIGH);
      }
    else if(goRight=1 && check == 1){
      Serial.println("go right");
      digitalWrite(dirPin, LOW);
      }
  }
  if((millis() - previousMillis)%200 < 3){
    readInputs();
  }

  
  if(check == 1 && ((goRight == 1 && switch_2_state==0) || (goLeft == 1 && switch_1_state==0))){

  digitalWrite(stepPin, HIGH);
  delayMicroseconds(30);
  digitalWrite(stepPin, LOW);
  delayMicroseconds(30);}
  
  
}
